@@ -22,13 +22,4 @@
 #undef GETTEXT_PACKAGE
 #undef GTKHTML_RELEASE
 #undef GTKHTML_RELEASE_STRING
-#undef GNOME_EXPLICIT_TRANSLATION_DOMAIN
-
-@BOTTOM@
-/* This is from libglade */
-
-#ifdef DEBUG
-#  define debug(stmnt) stmnt
-#else
-#  define debug(stmnt) /* nothing */
-#endif
+#undef HAVE_OLD_SOUP
