@@ -20,8 +20,6 @@
 
 #include "gtkhtml-editor-private.h"
 
-#include <libgnome/gnome-url.h>
-
 /******************************************************************************
  * Action Group Quick Reference
  *
@@ -141,19 +139,13 @@ insert_image_response_cb (GtkFileChooser *file_chooser,
                           gint response,
                           GtkhtmlEditor *editor)
 {
-	HTMLObject *image;
-	GtkHTML *html;
 	gchar *uri;
 
 	if (response != GTK_RESPONSE_OK)
 		return;
 
-	html = gtkhtml_editor_get_html (editor);
 	uri = gtk_file_chooser_get_uri (file_chooser);
-	image = html_image_new (
-		html_engine_get_image_factory (html->engine), uri,
-		NULL, NULL, 0, 0, 0, 0, 0, NULL, HTML_VALIGN_NONE, FALSE);
-	html_engine_paste_object (html->engine, image, 1);
+	gtkhtml_editor_insert_image (editor, uri);
 	g_free (uri);
 }
 
@@ -212,43 +204,6 @@ insert_html_file_response_cb (GtkFileChooser *file_chooser,
 	g_free (filename);
 }
 
-static void
-insert_emoticon (GtkhtmlEditor *editor,
-                 GtkAction *action,
-                 const gchar *alt)
-{
-	GtkHTML *html;
-	HTMLObject *image;
-	GtkIconInfo *icon_info;
-	const gchar *filename;
-	gchar *icon_name;
-	gchar *uri = NULL;
-
-	html = gtkhtml_editor_get_html (editor);
-
-	g_object_get (action, "icon-name", &icon_name, NULL);
-	icon_info = gtk_icon_theme_lookup_icon (
-		gtk_icon_theme_get_default (), icon_name, 16, 0);
-	g_free (icon_name);
-	g_return_if_fail (icon_info != NULL);
-
-	filename = gtk_icon_info_get_filename (icon_info);
-	if (filename != NULL)
-		uri = g_filename_to_uri (filename, NULL, NULL);
-	gtk_icon_info_free (icon_info);
-	g_return_if_fail (uri != NULL);
-
-	image = html_image_new (
-		html_engine_get_image_factory (html->engine),
-		uri, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
-		HTML_VALIGN_MIDDLE, FALSE);
-	html_image_set_alt (HTML_IMAGE (image), alt);
-	html_engine_paste_object (
-		html->engine, image, html_object_get_length (image));
-
-	g_free (uri);
-}
-
 static void
 insert_text_file_response_cb (GtkFileChooser *file_chooser,
                               gint response,
@@ -329,6 +284,9 @@ action_bold_cb (GtkToggleAction *action,
 {
 	const gchar *command;
 
+	if (editor->priv->ignore_style_change)
+		return;
+
 	if (gtk_toggle_action_get_active (action))
 		command = "bold-on";
 	else
@@ -625,6 +583,47 @@ action_indent_cb (GtkAction *action,
 	gtk_html_command (gtkhtml_editor_get_html (editor), "indent-more");
 }
 
+static void
+action_insert_face_cb (GtkhtmlFaceAction *action,
+                       GtkhtmlEditor *editor)
+{
+	GtkHTML *html;
+	HTMLObject *image;
+	GtkIconInfo *icon_info;
+	GtkIconTheme *icon_theme;
+	GtkhtmlFaceChooser *chooser;
+	GtkhtmlFace *face;
+	const gchar *filename;
+	gchar *uri = NULL;
+
+	html = gtkhtml_editor_get_html (editor);
+
+	chooser = GTKHTML_FACE_CHOOSER (action);
+	face = gtkhtml_face_chooser_get_current_face (chooser);
+	g_return_if_fail (face != NULL);
+
+	icon_theme = gtk_icon_theme_get_default ();
+	icon_info = gtk_icon_theme_lookup_icon (
+		icon_theme, face->icon_name, 16, 0);
+	g_return_if_fail (icon_info != NULL);
+
+	filename = gtk_icon_info_get_filename (icon_info);
+	if (filename != NULL)
+		uri = g_filename_to_uri (filename, NULL, NULL);
+	gtk_icon_info_free (icon_info);
+	g_return_if_fail (uri != NULL);
+
+	image = html_image_new (
+		html_engine_get_image_factory (html->engine),
+		uri, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
+		HTML_VALIGN_MIDDLE, FALSE);
+	html_image_set_alt (HTML_IMAGE (image), face->text_face);
+	html_engine_paste_object (
+		html->engine, image, html_object_get_length (image));
+
+	g_free (uri);
+}
+
 static void
 action_insert_html_file_cb (GtkToggleAction *action,
                             GtkhtmlEditor *editor)
@@ -665,125 +664,6 @@ action_insert_rule_cb (GtkAction *action,
 	gtk_action_activate (ACTION (PROPERTIES_RULE));
 }
 
-static void
-action_insert_face_angel_cb (GtkAction *action,
-                             GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, "O:-)");
-}
-
-static void
-action_insert_face_cool_cb (GtkAction *action,
-                            GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, "B-)");
-}
-
-static void
-action_insert_face_crying_cb (GtkAction *action,
-                              GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":'(");
-}
-
-static void
-action_insert_face_devilish_cb (GtkAction *action,
-                                GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ">:-)");
-}
-
-static void
-action_insert_face_embarrassed_cb (GtkAction *action,
-                                   GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":\"-)");
-}
-
-static void
-action_insert_face_kiss_cb (GtkAction *action,
-                            GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-*");
-}
-
-static void
-action_insert_face_monkey_cb (GtkAction *action,
-                              GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-(|)");
-}
-
-static void
-action_insert_face_plain_cb (GtkAction *action,
-                             GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-|");
-}
-
-static void
-action_insert_face_raspberry_cb (GtkAction *action,
-                                 GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-P");
-}
-
-static void
-action_insert_face_sad_cb (GtkAction *action,
-                           GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-(");
-}
-
-static void
-action_insert_face_smile_cb (GtkAction *action,
-                             GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-)");
-}
-
-static void
-action_insert_face_smile_big_cb (GtkAction *action,
-                                 GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-D");
-}
-
-static void
-action_insert_face_smirk_cb (GtkAction *action,
-                             GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-!");
-}
-
-static void
-action_insert_face_surprise_cb (GtkAction *action,
-                                GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-O");
-}
-
-static void
-action_insert_face_wink_cb (GtkAction *action,
-                            GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ";-)");
-}
-
-static void
-action_insert_smiley_9_cb (GtkAction *action,
-                           GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-/");
-}
-
-static void
-action_insert_smiley_26_cb (GtkAction *action,
-                            GtkhtmlEditor *editor)
-{
-	insert_emoticon (editor, action, ":-Q");
-}
-
 static void
 action_insert_table_cb (GtkAction *action,
                         GtkhtmlEditor *editor)
@@ -815,6 +695,9 @@ action_italic_cb (GtkToggleAction *action,
 {
 	const gchar *command;
 
+	if (editor->priv->ignore_style_change)
+		return;
+
 	if (gtk_toggle_action_get_active (action))
 		command = "italic-on";
 	else
@@ -856,6 +739,7 @@ action_language_cb (GtkToggleAction *action,
 	GtkhtmlSpellChecker *checker;
 	const gchar *language_code;
 	GtkAction *add_action;
+	GtkHTML *html;
 	GList *list;
 	guint length;
 	gchar *action_name;
@@ -897,6 +781,9 @@ action_language_cb (GtkToggleAction *action,
 	gtk_action_set_visible (ACTION (CONTEXT_SPELL_IGNORE), length > 0);
 
 	gtk_action_set_sensitive (ACTION (SPELL_CHECK), length > 0);
+
+	html = gtkhtml_editor_get_html (editor);
+	html_engine_spell_check (html->engine);
 }
 
 static void
@@ -968,6 +855,8 @@ action_mode_cb (GtkRadioAction *action,
 
 	html_engine_set_painter (html->engine, new_painter);
 	html_engine_schedule_redraw (html->engine);
+
+	g_object_notify (G_OBJECT (editor), "html-mode");
 }
 
 static void
@@ -978,6 +867,9 @@ action_monospaced_cb (GtkToggleAction *action,
 	GtkHTMLFontStyle and_mask;
 	GtkHTMLFontStyle or_mask;
 
+	if (editor->priv->ignore_style_change)
+		return;
+
 	if (gtk_toggle_action_get_active (action)) {
 		and_mask = GTK_HTML_FONT_STYLE_MAX;
 		or_mask = GTK_HTML_FONT_STYLE_FIXED;
@@ -1074,6 +966,9 @@ action_style_cb (GtkRadioAction *action,
 {
 	const gchar *command = NULL;
 
+	if (editor->priv->ignore_style_change)
+		return;
+
 	switch (gtk_radio_action_get_current_value (current)) {
 		case GTK_HTML_PARAGRAPH_STYLE_NORMAL:
 			command = "style-normal";
@@ -1160,6 +1055,9 @@ action_size_cb (GtkRadioAction *action,
 {
 	const gchar *command = NULL;
 
+	if (editor->priv->ignore_style_change)
+		return;
+
 	switch (gtk_radio_action_get_current_value (current)) {
 		case GTK_HTML_FONT_STYLE_SIZE_1:
 			command = "size-minus-2";
@@ -1200,6 +1098,9 @@ action_strikethrough_cb (GtkToggleAction *action,
 {
 	const gchar *command;
 
+	if (editor->priv->ignore_style_change)
+		return;
+
 	if (gtk_toggle_action_get_active (action))
 		command = "strikeout-on";
 	else
@@ -1212,13 +1113,17 @@ static void
 action_test_url_cb (GtkAction *action,
                     GtkhtmlEditor *editor)
 {
+	GtkEntry *entry;
+	GtkWindow *window;
 	const gchar *text;
 
-	text = gtk_entry_get_text (
-		GTK_ENTRY (WIDGET (LINK_PROPERTIES_URL_ENTRY)));
+	entry = GTK_ENTRY (WIDGET (LINK_PROPERTIES_URL_ENTRY));
+	window = GTK_WINDOW (WIDGET (LINK_PROPERTIES_WINDOW));
+
+	text = gtk_entry_get_text (entry);
 
 	if (text != NULL && *text != '\0')
-		gnome_url_show (text, NULL);
+		gtkhtml_editor_show_uri (window, text);
 }
 
 static void
@@ -1227,6 +1132,9 @@ action_underline_cb (GtkToggleAction *action,
 {
 	const gchar *command;
 
+	if (editor->priv->ignore_style_change)
+		return;
+
 	if (gtk_toggle_action_get_active (action))
 		command = "underline-on";
 	else
@@ -1341,125 +1249,6 @@ static GtkActionEntry core_entries[] = {
 	  NULL,
 	  G_CALLBACK (action_insert_html_file_cb) },
 
-	{ "insert-face-angel",
-	  "face-angel",
-	  N_("_Angel"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_angel_cb) },
-
-	{ "insert-face-cool",
-	  "face-cool",
-	  N_("_Cool"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_cool_cb) },
-
-	{ "insert-face-crying",
-	  "face-crying",
-	  N_("Cr_ying"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_crying_cb) },
-
-	{ "insert-face-devilish",
-	  "face-devilish",
-	  N_("_Devilish"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_devilish_cb) },
-
-	{ "insert-face-embarrassed",
-	  "face-embarrassed",
-	  N_("_Embarrassed"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_embarrassed_cb) },
-
-	{ "insert-face-kiss",
-	  "face-kiss",
-	  N_("_Kiss"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_kiss_cb) },
-
-	{ "insert-face-monkey",
-	  "face-monkey",
-	  N_("_Monkey"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_monkey_cb) },
-
-	{ "insert-face-plain",
-	  "face-plain",
-	  N_("_Indifferent"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_plain_cb) },
-
-	{ "insert-face-raspberry",
-	  "face-raspberry",
-	  N_("_Tongue"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_raspberry_cb) },
-
-	{ "insert-face-sad",
-	  "face-sad",
-	  N_("_Frown"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_sad_cb) },
-
-	{ "insert-face-smile",
-	  "face-smile",
-	  N_("_Smile"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_smile_cb) },
-
-	{ "insert-face-smile-big",
-	  "face-smile-big",
-	  N_("_Laughing"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_smile_big_cb) },
-
-	{ "insert-face-smirk",
-	  "face-smirk",
-	  N_("Smi_rk"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_smirk_cb) },
-
-	{ "insert-face-surprise",
-	  "face-surprise",
-	  N_("Sur_prised"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_surprise_cb) },
-
-	{ "insert-face-wink",
-	  "face-wink",
-	  N_("_Wink"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_face_wink_cb) },
-
-	{ "insert-smiley-9",
-	  "stock_smiley-9",
-	  N_("_Undecided"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_smiley_9_cb) },
-
-	{ "insert-smiley-26",
-	  "stock_smiley-26",
-	  N_("S_ick"),
-	  NULL,
-	  NULL,
-	  G_CALLBACK (action_insert_smiley_26_cb) },
-
 	{ "insert-text-file",
 	  NULL,
 	  N_("Te_xt File..."),
@@ -1540,7 +1329,7 @@ static GtkActionEntry core_entries[] = {
 	{ "wrap-lines",
 	  NULL,
 	  N_("_Wrap Lines"),
-	  "<Control>backslash",
+	  "<Control>k",
 	  NULL,
 	  G_CALLBACK (action_wrap_lines_cb) },
 
@@ -1574,13 +1363,6 @@ static GtkActionEntry core_entries[] = {
 	  NULL,
 	  NULL },
 
-	{ "insert-face-menu",
-	  NULL,
-	  N_("_Emoticon"),
-	  NULL,
-	  NULL,
-	  NULL },
-
 	{ "insert-menu",
 	  NULL,
 	  N_("_Insert"),
@@ -2149,6 +1931,15 @@ static GtkActionEntry spell_context_entries[] = {
 	  N_("Add Word To"),
 	  NULL,
 	  NULL,
+	  NULL },
+
+	/* Menus */
+
+	{ "context-more-suggestions-menu",
+	  NULL,
+	  N_("More Suggestions"),
+	  NULL,
+	  NULL,
 	  NULL }
 };
 
@@ -2278,6 +2069,7 @@ editor_actions_setup_spell_check_menu (GtkhtmlEditor *editor)
 void
 gtkhtml_editor_actions_init (GtkhtmlEditor *editor)
 {
+	GtkAction *action;
 	GtkActionGroup *action_group;
 	GtkUIManager *manager;
 	const gchar *domain;
@@ -2310,6 +2102,16 @@ gtkhtml_editor_actions_init (GtkhtmlEditor *editor)
 		G_CALLBACK (action_style_cb), editor);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 
+	/* Face Action */
+	action = gtkhtml_face_action_new (
+		"insert-face", _("_Emoticon"),
+		_("Insert Emoticon"), NULL);
+	g_object_set (action, "icon-name", "face-smile", NULL);
+	g_signal_connect (
+		action, "item-activated",
+		G_CALLBACK (action_insert_face_cb), editor);
+	gtk_action_group_add_action (action_group, action);
+
 	/* Core Actions (HTML only) */
 	action_group = editor->priv->html_actions;
 	gtk_action_group_set_translation_domain (action_group, domain);
