@@ -46,7 +46,7 @@ get_locale_language (void)
 	locale = g_win32_getlocale ();
 #endif
 	if (locale && strcmp (locale, "C") && strcmp (locale, "POSIX")) {
-		char *codeset, *lang;
+		gchar *codeset, *lang;
 
 		if ((codeset = strchr (locale, '.')))
 			lang = g_strndup (locale, codeset - locale);
@@ -62,7 +62,7 @@ get_locale_language (void)
 
 				/* validate the country code */
 				if (strlen (lang + 3) > 2) {
-				        /* invalid country code */
+					/* invalid country code */
 					lang[2] = '\0';
 				} else {
 					lang[2] = '-';
